@@ -2,7 +2,7 @@ package RedisDB;
 
 use strict;
 use warnings;
-our $VERSION = "2.21_01";
+our $VERSION = "2.31_01";
 $VERSION = eval $VERSION;
 
 use RedisDB::Error;
@@ -196,7 +196,7 @@ sub _on_connect_error {
     my $server = $self->{path} || ("$self->{host}:$self->{port}");
     my $error_obj =
       RedisDB::Error::DISCONNECTED->new("Couldn't connect to the redis server at $server: $!");
-    confess $error_obj;
+    die $error_obj;
 }
 
 sub _on_disconnect {
@@ -286,19 +286,21 @@ sub _connect {
     }
     continue {
         unless ( $self->{_socket} ) {
-            if ( $self->{raise_error} ) {
+            my $new_error;
+            try {
                 $self->{on_connect_error}->( $self, $error );
             }
-            else {
-                my $new_error = try {
-                    $self->{on_connect_error}->( $self, $error );
-                    return 0;
+            catch {
+                if ( $self->{raise_error} ) {
+                    $self->reset_connection;
+                    die $_;
                 }
-                catch {
-                    return $_;
-                };
-                return $new_error if $new_error;
-            }
+                else {
+                    $self->{_parser}->propagate_reply($_) if $self->{_parser};
+                    $new_error = $_;
+                }
+            };
+            return $new_error if $new_error;
         }
     }
 
@@ -693,6 +695,25 @@ sub get_reply {
         croak $res;
     }
 
+    if ( $self->{_subscription_loop} ) {
+        confess "Expected multi-bulk reply, but got $res" unless ref $res;
+        if ( $res->[0] eq 'message' ) {
+            $self->{_subscribed}{ $res->[1] }( $self, $res->[1], undef, $res->[2] )
+              if $self->{_subscribed}{ $res->[1] };
+        }
+        elsif ( $res->[0] eq 'pmessage' ) {
+            $self->{_psubscribed}{ $res->[1] }( $self, $res->[2], $res->[1], $res->[3] )
+              if $self->{_psubscribed}{ $res->[1] };
+        }
+        elsif ( $res->[0] =~ /^p?(un)?subscribe/ ) {
+
+            # ignore
+        }
+        else {
+            confess "Got unknown reply $res->[0] in subscription mode";
+        }
+    }
+
     return $res;
 }
 
@@ -773,26 +794,27 @@ sub version {
 
 # don't forget to update POD
 my @commands = qw(
-  append	auth	bgrewriteaof	bgsave	bitcount	bitop
+  append	auth	bgrewriteaof	bgsave	bitcount	bitop	bitpos
   blpop	brpop   brpoplpush	client	client_kill	client_getname	client_setname
-  config	config_get	config_set	config_resetstat
+  config	config_get	config_set	config_resetstat config_rewrite
   dbsize	debug_object	debug_segfault
   decr	decrby	del	dump	echo	eval    evalsha exists	expire	expireat	flushall
   flushdb	get	getbit	getrange	getset	hdel	hexists	hget	hgetall
-  hincrby	hincrbyfloat	hkeys	hlen	hmget	hmset	hset	hsetnx	hvals	incr	incrby
+  hincrby	hincrbyfloat	hkeys	hlen	hmget	hscan	hmset	hset	hsetnx	hvals	incr	incrby
   incrbyfloat	keys	lastsave	lindex	linsert	llen	lpop	lpush	lpushx
   lrange	lrem	lset	ltrim	mget	migrate	move	mset	msetnx	object	object_refcount
   object_encoding	object_idletime	persist	pexpire	pexpireat	ping	psetex	pttl
+  pubsub	pubsub_channels	pubsub_numsub	pubsub_numpat
   publish	quit	randomkey	rename	renamenx	restore	rpop	rpoplpush
-  rpush	rpushx	sadd	save	scard	script	script_exists   script_flush    script_kill
+  rpush	rpushx	sadd	save	scan	scard	script	script_exists   script_flush    script_kill
   script_load   sdiff	sdiffstore	select	set
   setbit	setex	setnx	setrange	sinter	sinterstore
   sismember	slaveof	slowlog smembers	smove	sort	spop	srandmember
-  srem	strlen	sunion	sunionstore	sync	time    ttl	type	unwatch watch
+  srem	sscan	strlen	sunion	sunionstore	sync	time    ttl	type	unwatch watch
   zadd	zcard
   zcount	zincrby	zinterstore	zrange	zrangebyscore	zrank	zrem
   zremrangebyrank   zremrangebyscore	zrevrange	zrevrangebyscore	zrevrank
-  zscore	zunionstore
+  zscan	zscore	zunionstore
 );
 
 sub _simple_commands {
@@ -819,23 +841,24 @@ I<execute>, waits for the reply from the server, and returns it. E.g.:
     $redis->send_command("get", $key, sub { $val = $_[1] });
 
 The following wrapper methods are defined: append, auth, bgrewriteaof, bgsave,
-bitcount, bitop, blpop, brpop, brpoplpush, client, client_kill, client_getname,
-client_setname, config, config_get, config_set, config_resetstat, dbsize,
+bitcount, bitop, bitpos, blpop, brpop, brpoplpush, client, client_kill, client_getname,
+client_setname, config, config_get, config_set, config_resetstat, config_rewrite, dbsize,
 debug_object, debug_segfault, decr, decrby, del, dump, echo, eval, evalsha,
 exists, expire, expireat, flushall, flushdb, get, getbit, getrange, getset,
-hdel, hexists, hget, hgetall, hincrby, hincrbyfloat, hkeys, hlen, hmget, hmset,
+hdel, hexists, hget, hgetall, hincrby, hincrbyfloat, hkeys, hlen, hmget, hscan, hmset,
 hset, hsetnx, hvals, incr, incrby, incrbyfloat, keys, lastsave, lindex,
 linsert, llen, lpop, lpush, lpushx, lrange, lrem, lset, ltrim, mget, migrate,
 move, mset, msetnx, object, object_refcount, object_encoding, object_idletime,
-persist, pexpire, pexpireat, ping, psetex, pttl, publish, quit, randomkey,
-rename, renamenx, restore, rpop, rpoplpush, rpush, rpushx, sadd, save, scard,
+persist, pexpire, pexpireat, ping, psetex, pttl, publish,
+pubsub, pubsub_channels, pubsub_numsub, pubsub_numpat, quit, randomkey,
+rename, renamenx, restore, rpop, rpoplpush, rpush, rpushx, sadd, save, scan, scard,
 script, script_exists, script_flush, script_kill, script_load, sdiff,
 sdiffstore, select, set, setbit, setex, setnx, setrange, sinter, sinterstore,
 sismember, slaveof, slowlog, smembers, smove, sort, spop, srandmember, srem,
-strlen, sunion, sunionstore, sync, time, ttl, type, unwatch, watch, zadd,
+sscan strlen, sunion, sunionstore, sync, time, ttl, type, unwatch, watch, zadd,
 zcard, zcount, zincrby, zinterstore, zrange, zrangebyscore, zrank, zrem,
 zremrangebyrank, zremrangebyscore, zrevrange, zrevrangebyscore, zrevrank,
-zscore, zunionstore.
+zscan, zscore, zunionstore.
 
 See description of all commands in redis documentation at
 L<http://redis.io/commands>.
@@ -941,6 +964,68 @@ sub shutdown {
     return;
 }
 
+=head2 $self->scan_all([MATCH => $pattern,][COUNT => $count,])
+
+this method starts a new SCAN iteration and executes SCAN commands till cursor
+returned by server is 0. It then returns all the keys returned by server during
+the iteration. MATCH and COUNT are passed to SCAN command. In case of success
+returns reference to array with matching keys, in case of error dies or returns
+L<RedisDB::Error> object depending on I<raise_error> option.
+
+=cut
+
+sub scan_all {
+    my $self = shift;
+    if ( ref $_[-1] eq 'CODE' ) {
+        croak "scan_all does not accept callback parameter";
+    }
+    my $cursor = 0;
+    my @result;
+    do {
+        my $res = $self->execute( 'SCAN', $cursor, @_ );
+
+        # in case of error just return it
+        return $res unless ref $res eq 'ARRAY';
+        $cursor = $res->[0];
+        push @result, @{ $res->[1] };
+    } while $cursor;
+    return \@result;
+}
+
+=head2 $self->hscan_all($key, [MATCH => $pattern,][COUNT => $count,])
+
+=head2 $self->sscan_all($key, [MATCH => $pattern,][COUNT => $count,])
+
+=head2 $self->zscan_all($key, [MATCH => $pattern,][COUNT => $count,])
+
+these three methods are doing the same thing as I<scan_all> except that they
+require a key as the first parameter, and they iterate using HSCAN, SSCAN and
+ZSCAN commands.
+
+=cut
+
+for my $command (qw(hscan sscan zscan)) {
+    my $uccom = uc $command;
+    no strict 'refs';
+    my $name = "${command}_all";
+    *{ __PACKAGE__ . "::$name" } = sub {
+        my $self = shift;
+        my $key  = shift;
+        if ( ref $_[-1] eq 'CODE' ) {
+            croak "$name does not accept callback parameter";
+        }
+        my $cursor = 0;
+        my @result;
+        do {
+            my $res = $self->execute( $uccom, $key, $cursor, @_ );
+            return $res unless ref $res eq 'ARRAY';
+            $cursor = $res->[0];
+            push @result, @{ $res->[1] };
+        } while $cursor;
+        return \@result;
+    };
+}
+
 =head1 UTF-8 SUPPORT
 
 The redis protocol is designed to work with the binary data, both keys and
@@ -1082,24 +1167,24 @@ you can't use any other redis commands like set, get, etc. Here is the example
 of running in the subscription mode:
 
     my $message_cb = sub {
-        my ($redis, $channel, $pattern, $message) = @_;
+        my ( $redis, $channel, $pattern, $message ) = @_;
         print "$channel: $message\n";
     };
-    
+
     my $control_cb = sub {
-        my ($redis, $channel, $pattern, $message) = @_;
-        if ($channel eq 'control.quit') {
+        my ( $redis, $channel, $pattern, $message ) = @_;
+        if ( $channel eq 'control.quit' ) {
             $redis->unsubscribe;
             $redis->punsubscribe;
         }
-        elsif ($channel eq 'control.subscribe') {
+        elsif ( $channel eq 'control.subscribe' ) {
             $redis->subscribe($message);
         }
     };
-    
-    subscription_loop(
-        subscribe => [ 'news',  ],
-        psubscribe => [ 'control.*' => $control_cb ],
+
+    $redis->subscription_loop(
+        subscribe        => [ 'news', ],
+        psubscribe       => [ 'control.*' => $control_cb ],
         default_callback => $message_cb,
     );
 
@@ -1159,11 +1244,11 @@ for every channel you are going to subscribe.
 
 sub subscription_loop {
     my ( $self, %args ) = @_;
-    croak "Already in subscription loop" if $self->{_subscription_loop};
+    croak "Already in subscription loop" if $self->{_subscription_loop} > 0;
     croak "You can't start subscription loop while in pipelining mode."
       if $self->replies_to_fetch;
-    $self->{_subscribed}        = {};
-    $self->{_psubscribed}       = {};
+    $self->{_subscribed}  ||= {};
+    $self->{_psubscribed} ||= {};
     $self->{_subscription_cb}   = $args{default_callback};
     $self->{_subscription_loop} = 1;
     $self->{_parser}->set_default_callback( \&_queue );
@@ -1186,23 +1271,7 @@ sub subscription_loop {
       unless ( keys %{ $self->{_subscribed} } or keys %{ $self->{_psubscribed} } );
 
     while ( $self->{_subscription_loop} ) {
-        my $msg = $self->get_reply;
-        confess "Expected multi-bulk reply, but got $msg" unless ref $msg;
-        if ( $msg->[0] eq 'message' ) {
-            $self->{_subscribed}{ $msg->[1] }( $self, $msg->[1], undef, $msg->[2] )
-              if $self->{_subscribed}{ $msg->[1] };
-        }
-        elsif ( $msg->[0] eq 'pmessage' ) {
-            $self->{_psubscribed}{ $msg->[1] }( $self, $msg->[2], $msg->[1], $msg->[3] )
-              if $self->{_psubscribed}{ $msg->[1] };
-        }
-        elsif ( $msg->[0] =~ /^p?(un)?subscribe/ ) {
-
-            # ignore
-        }
-        else {
-            confess "Got unknown reply $msg->[0] in subscription mode";
-        }
+        $self->get_reply;
     }
     return;
 }
@@ -1228,10 +1297,10 @@ sub subscribe {
           or croak "Callback for $channel not specified, neither default callback defined";
     }
     else {
-        $callback = 1;
+        $callback ||= sub { 1 };
     }
     $self->{_subscribed}{$channel} = $callback;
-    $self->send_command( "SUBSCRIBE", $channel );
+    $self->send_command( "SUBSCRIBE", $channel, \&_queue );
     return;
 }
 
@@ -1256,10 +1325,10 @@ sub psubscribe {
           or croak "Callback for $channel not specified, neither default callback defined";
     }
     else {
-        $callback = 1;
+        $callback ||= sub { 1 };
     }
     $self->{_psubscribed}{$channel} = $callback;
-    $self->send_command( "PSUBSCRIBE", $channel );
+    $self->send_command( "PSUBSCRIBE", $channel, \&_queue );
     return;
 }
 
@@ -1279,7 +1348,9 @@ sub unsubscribe {
     else {
         $self->{_subscribed} = {};
     }
-    if ( %{ $self->{_subscribed} } or %{ $self->{_psubscribed} } ) {
+    if (   %{ $self->{_subscribed} }
+        or %{ $self->{_psubscribed} || {} } )
+    {
         return $self->send_command( "UNSUBSCRIBE", @_ );
     }
     else {
@@ -1305,7 +1376,7 @@ sub punsubscribe {
     else {
         $self->{_psubscribed} = {};
     }
-    if (   %{ $self->{_subscribed} }
+    if (   %{ $self->{_subscribed} || {} }
         or %{ $self->{_psubscribed} } )
     {
         return $self->send_command( "PUNSUBSCRIBE", @_ );
@@ -1445,7 +1516,7 @@ Pavel Shaydo, C<< <zwon at cpan.org> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2011-2013 Pavel Shaydo.
+Copyright 2011-2014 Pavel Shaydo.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
